# Summary

- [Introduction](./intro.md)
- [Idioms](./idioms/index.md)
  - [Use borrowed types for arguments](./idioms/coercion-arguments.md)
  - [Concatenating Strings with `format!`](./idioms/concat-format.md)
  - [Constructor](./idioms/ctor.md)
  - [The `Default` Trait](./idioms/default.md)
  - [Collections Are Smart Pointers](./idioms/deref.md)
  - [Finalisation in Destructors](./idioms/dtor-finally.md)
  - [`mem::{take(_), replace(_)}`](./idioms/mem-replace.md)
  - [On-Stack Dynamic Dispatch](./idioms/on-stack-dyn-dispatch.md)
  - [Foreign function interface usage](./idioms/ffi-intro.md)
    - [Idiomatic Errors](./idioms/ffi-errors.md)
    - [Accepting Strings](./idioms/ffi-accepting-strings.md)
    - [Passing Strings](./idioms/ffi-passing-strings.md)
  - [Iterating over an `Option`](./idioms/option-iter.md)
  - [Pass Variables to Closure](./idioms/pass-var-to-closure.md)
  - [Privacy For Extensibility](./idioms/priv-extend.md)
  - [Easy doc initialization](./idioms/rustdoc-init.md)
  - [Temporary mutability](./idioms/temporary-mutability.md)

- [Design Patterns](./patterns/index.md)
  - [Builder](./patterns/builder.md)
  - [Compose Structs](./patterns/compose-structs.md)
  - [Entry API](./patterns/entry.md)
  - [Foreign function interface usage](./patterns/ffi-intro.md)
    - [Object-Based APIs](./patterns/ffi-export.md)
    - [Type Consolidation into Wrappers](./patterns/ffi-wrappers.md)
  - [Fold](./patterns/fold.md)
  - [Newtype](./patterns/newtype.md)
  - [RAII Guards](./patterns/RAII.md)
  - [Prefer Small Crates](./patterns/small-crates.md)
  - [Strategy](./patterns/strategy.md)
  - [Contain unsafety in small modules](./patterns/unsafe-mods.md)
  - [Visitor](./patterns/visitor.md)

- [Anti-patterns](./anti_patterns/index.md)
  - [`#[deny(warnings)]`](./anti_patterns/deny-warnings.md)
  - [Deref Polymorphism](./anti_patterns/deref.md)

- [Functional Programming](./functional/index.md)
  - [Programming paradigms](./functional/paradigms.md)

- [Additional Resources](./additional_resources/index.md)
  - [Design principles](./additional_resources/design-principles.md)
